------------------------------------------
-- File:		Common.lua
-- Description:
-- Date:		20/01/10 10:29 PM
--------------------------------------

-- Automatically included

-----------------------------------------------------------------------
-- Common math
function clamp(value, minLimit, maxLimit)
	assert(maxLimit >= maxLimit)
	if (value < minLimit) then
		return minLimit
	end
	if (value > maxLimit) then
		return maxLimit
	end
	return value
end

function saturate(value)
	return clamp(value, 0, 1)
end

function lerp(a, b, t)
	return a * (1.0 - t) + b * t
end

-- Constants
EPSILON = 0.00001
NUMBER_MAX = 1e32

-----------------------------------------------------------------------
-- 2D  math
function m2MakeVec(x, y)
	return {x = x, y = y}
end

function m2MakePerpCCW(a)
	return m2MakeVec(-a.y, a.x)
end

function m2MakePerpCW(a)
	return m2MakeVec(a.y, -a.x)
end

-----------------------------------------------------------------------
-- 3D  math
function m3MakeVec(x, y, z)
	return {x = x, y = y, z = z}
end

function m3MakeZero()
	return m3MakeVec(0, 0, 0)
end

function m3MakeUp()
	return m3MakeVec(0, 1, 0)
end

function m3CopyVec(a)
	return m3MakeVec(a.x, a.y, a.z)
end

function m3Sum(a, b)
	return m3MakeVec(a.x + b.x, a.y + b.y, a.z + b.z)
end

function m3Add(a, b)
	a.x = a.x + b.x
	a.y = a.y + b.y
	a.z = a.z + b.z
end
function m3AddMake(a, b)
	x = a.x + b.x
	y = a.y + b.y
	z = a.z + b.z
	return m3MakeVec(x,y,z)
end
function m3Diff(a, b)
	return m3MakeVec(a.x - b.x, a.y - b.y, a.z - b.z)
end

function m3Sub(a, b)
	a.x = a.x - b.x
	a.y = a.y - b.y
	a.z = a.z - b.z
end

function m3Cross(a, b)
	return m3MakeVec(a.y * b.z - b.y * a.z, a.z * b.x - b.z * a.x, a.x * b.y + b.x * a.y)
end

function m3Length(a)
	return math.sqrt(a.x * a.x + a.y * a.y + a.z * a.z)
end

function m3Negate(a)
	a.x = -a.x
	a.y = -a.y
	a.z = -a.z
end

function m3Mul(a, scalar)
	a.x = a.x * scalar
	a.y = a.y * scalar
	a.z = a.z * scalar
end

function m3MulMake(a, scalar)
    return m3MakeVec(a.x * scalar, a.y * scalar, a.z * scalar)
end

function m3Div(a, scalar)
	a.x = a.x / scalar
	a.y = a.y / scalar
	a.z = a.z / scalar
end

function m3Dot(a, b)
	return a.x * b.x + a.y * b.y + a.z * b.z
end

function m3Normalize(a)
	local l = m3Length(a)
	m3Div(a, l)
end

function m3Lerp(a, b, t)
	return m3MakeVec(lerp(a.x, b.x, t), lerp(a.y, b.y, t), lerp(a.z, b.z, t))
end

function m3Print(a)
	trace("("..a.x..";"..a.y..";"..a.z..")")
end

-----------------------------------------------------------------------
-- Planes
function m3PlaneDist(v, plane)
	return v.x * plane.x + v.y * plane.y + v.z * plane.z + plane.w
end

-----------------------------------------------------------------------
-- Matrices
function m3BuildMatrix(tmRightO, tmUpO, tmOrg)	
	local tmDirO = m3Cross(tmRightO, tmUpO)
	return {
		_11 = tmRightO.x, 	_12 = tmRightO.y, 	_13 = tmRightO.z, 	_14 = 0,
		_21 = tmUpO.x, 		_22 = tmUpO.y, 		_23 = tmUpO.z, 		_24 = 0,
		_31 = tmDirO.x, 	_32 = tmDirO.y,		_33 = tmDirO.z, 	_34 = 0,
		_41 = tmOrg.x, 		_42 = tmOrg.y, 		_43 = tmOrg.z, 		_44 = 0}
end

function m3Transform(v, tm)
	return m3MakeVec(
		v.x * tm._11 + v.y * tm._21 + v.z * tm._31 + tm._41, 
		v.x * tm._12 + v.y * tm._22 + v.z * tm._32 + tm._42,
		v.x * tm._13 + v.y * tm._23 + v.z * tm._33 + tm._43)
end

function m3TransformNormal(v, tm)
	-- should compute transpose of the inverse matrix
	return m3MakeVec(
		v.x * tm._11 + v.y * tm._21 + v.z * tm._31, 
		v.x * tm._12 + v.y * tm._22 + v.z * tm._32,
		v.x * tm._13 + v.y * tm._23 + v.z * tm._33)
end

-----------------------------------------------------------------------
-- More math
function ToDegree(a)
	return a / (2 * math.pi) * 360
end

function ToRadians(a)
	return a / 360.0 * 2 * math.pi
end

function To360(a)
	while (a > 2 * math.pi) do
		a = a - 2 * math.pi
	end
	while (a < 0) do
		a = a + 2 * math.pi
	end
	return a
end

function GetAngleDifference(a, b)
	a = To360(a)
	b = To360(b)

	local difference = nil
	if (a > b) then
		difference = a - b
	else
		difference = b - a
	end

	if (difference > math.pi) then
		difference = 2 * math.pi - difference
	end

	return difference
end


function IsAngleDifferenceCW(a, b)
	a = To360(a)
	b = To360(b)

	local difference = b - a
	if ((difference > 0 and difference < math.pi) or (difference < -math.pi and difference > - 2 * math.pi)) then
		return false
	end

	return true
end

-----------------------------------------------------------------------
function GetNumEmitParticles(minEmitInterval, emitAmount, elapsedTime)
    if (minEmitInterval < EPSILON or emitAmount < EPSILON) then
        return 0
    end
    local numParticles = 0
    if (emitAmount > EPSILON) then
        local emitInterval = minEmitInterval / emitAmount

        numParticles = elapsedTime / emitInterval
		--numParticles = (elapsedTime * emitAmmout )/ minEmitInterval
        if (numParticles < 1.0) then
            if (math.random() < numParticles) then
                numParticles = 1.0
            else
				numParticles = 0
			end
		end
    end
    return numParticles
end
-----------------------------------------------------------------------
--EOF
